
% matlab version of pacbb with its driver sample
%
%--------------------------------------------------------------------------
% this is a driver to use the projected adaptive CBB method 
% to minimize nonlinear function f(x) with box and one linear constraits
%
%     min  f(x)
%
%    s.t. BL <= x <= BU
%
%          r^T x = b
%
%    where f(x): \Re^{n} -> R is a continuous differentialble function, 
%
%    BL, BU and r are given vectors in \Re^{n}, b is a scalar
%
%-------------------------------------------------------------------------



% give the dimention n 

n = 20;

% give the data r, b 

r = ones(n,1);
b = 20.;

%set the initial guess

x = zeros(n,1);

for i = 1:n
    x(i) = i + 1.;
end

% set up lower and upper bounds

 BL = zeros(n,1);
 BU = 2.0 * ones(n,1);

% check the feasible region

b1 = 0;
b2 = 0;
for i = 1:n
    t = r(i);
    if (t >= 0.)
            b1 = b1 + t*BU(i);
            b2 = b2 + t*BL(i);
    else
        b1 = b1 + t*BL(i);
        b2 = b2 + t*BU(i);   
    end
end
  
if ( b1 < b || b2 > b )
    disp('the feasible reagion is empty!'); 
    return;
end

% initial parameter set up

% eps is the stopping tolerance,
% setting neps = 1, the algorithm stops when pgnorm <= eps,
% setting neps = 0, the algorithm stops when pgnorm <= eps*(1 + f),
% where pgnorm and f are projected gradient norm and function value at x_k.

eps = 1.d-6;       
neps = 1;

[flag, nfunc, ngrad, iter, pgnorm, f, x] = pacbb(n, x, r, b, BL, BU, eps, neps);

% print the final statistics
fprintf("\n On the return:");
fprintf("\n Iter: %4i f: %20.15f pgnorm: %20.15f nfunc %4i ngrad %4i flag: %i \n", ...
          iter, f, pgnorm, nfunc, ngrad, flag);
      
% check the feasibility of the solution  
    
t = 0.;

for i=1:n

    if ( x(i) < BL(i) || x(i) > BU(i) )
    
        disp('the solution is not feasible!');
        disp(['x(' sprintf('%4i',i) ')= ' sprintf('%10.6f',x(i)) ' is not feasible']);
    end
    t = t + r(i)*x(i);
end

disp([' r^T x - b = ' sprintf('%10.6f', t-b)]);



