
function [flag, nfunc, ngrad, iter, pgnorm, f, x] = ...
          pacbb(n, x0, r, b, BL, BU, eps, neps)
      
      
% x = zeros(n,1);
% d = zeros(n,1);
% g = zeros(n,1);
% pg = zeros(n,1);
 xtemp = zeros(n,1);
% gtemp= zeros(n,1);

 
% If it is much more efficient to calculate the function and gradient
% values simultaneously, set fg_provide = 1 and provide the subroutine
% evalfg to calculate the function and gradient simultaneously; 
% Otherwise, set fg_provide = 0 and provide the subroutines evalf and evalg
% to calculate the function and gradient individually. 

fg_provide = 1;

x = x0;
maxit = 10000;
maxfunc = 99999;
StopRule = neps;
user_tol = eps;
iter = 0;
nfunc = 0;
ngrad = 0;
inform = 0;
flag = 0;

[~, m, ~, nm, ~, ~, ~, ~, gamma, ...
 LMIN, LMAX, tolx] = init_cbbparm1 (n);
      
%----------------------------------------------- 
% project initial guess on the feasible region
%-----------------------------------------------

[x, inform] = proj(n, x, r, b, BL, BU);

xbest = x;

if (inform > 0 )
    disp('Error in the projection!');
    flag = 6;
    return;
end

if ( fg_provide == 1 )
    [f, g, nfunc, ngrad] = evalfg (n, x, nfunc, ngrad);
else
    [f, nfunc] = evalf (n, x, nfunc);
    [g, ngrad] = evalg (n, x, ngrad);
end

lastfvalues = zeros(m,1);
lastfvalues(1) = f;

for i=2:m
    lastfvalues(i) = -1.0e99;
end

mm = 0;    
ll = 0;     
nl = 0;    
np = 0;                    
fmin = f;    
fr = f;     
fc = f;            

pg = x - g ;

[pg, inform] = proj(n, pg, r, b, BL, BU);

if (inform > 0 )
    disp('Error in the projection!');
    flag = 6;
    return;
end

pg = pg - x;

pgnorm = max(abs(pg));
pgnorm_best = pgnorm;

disp([' It.: ' sprintf('%4i',iter) ' Obj.: ' sprintf('%20.15f',f) ...
      ' pgnorm.: ' sprintf('%20.15f', pgnorm)]);

[TestTol1] = TestTol (f, pgnorm, user_tol, StopRule);
   
if ( TestTol1 == 1 )
    return;  
else
    xnorm = max(abs(x));
    
    if (xnorm ~= 0.0)
       lambda = xnorm/pgnorm;    
    else
        lambda = 1.0/pgnorm;
    end            
end

while (1==1)

    [TestTol1] = TestTol (f, pgnorm, user_tol, StopRule);

    if (TestTol1 == 1)
        return;
    end
    
    if ( iter >= maxit || nfunc >= maxfunc )    
        flag = 10;     
        return;
    end
    
    iter = iter + 1;

    d = x - lambda*g;
    [d, inform] = proj(n, d, r, b, BL, BU);

    if ( inform > 0 )
        disp('Error in the projection!');
        flag = 6;  
        return;
    end
    
    hitbound = 0;
    sts = 0.0;
    gtd = 0.0;
    
    for i=1:n

        t = d(i);
        xtemp(i) = t;
        t1 = t - x(i);
        
        if (t >= BU(i))
            
            t = BU(i) - x(i);  
            if ( t >= tolx ) 
                hitbound = 1;
            end
            
        elseif ( t <= BL(i))
            
            t = BL(i) - x(i);
            if ( -t >= tolx )
                hitbound = 1;   
            end
        end
        
        d(i) = t1;   
        gtd = gtd + g(i)*t1;
        sts = sts + t1*t1;
    end
    
    if (hitbound==1)
        ll = ll+1;
    end
    
    nfunco = nfunc;
    
    if ( gtd >= 0.0)
        
        disp('Error: searching direction is not descent!');
        return;
    end
    
    [ftemp, gtemp, xtemp, nfunc, ngrad, lastfvalues, alpha, flag, ...
    fmin, fr, fc, nl, np, mm, ll] = ...
    cbbls1(n, x, f, d, gtd, xtemp, nfunc, ngrad, lastfvalues, flag, ...
    fmin, fr, fc, nl, np, mm, ll, fg_provide);
    
    if ( flag > 0 )
        disp('Error: line search fail!');
        break;
    end
    
    sts = sts*alpha*alpha;
    
    if ( nfunc == (nfunco+1))
        np = np + 1;
    else
        np = 0;
    end
    
    if ( fg_provide ~= 1 )
        [gtemp, ngrad] = evalg(n, xtemp, ngrad);     
    end
    index = 0;  
   
    if ( (ll >= 1) || (mm >= nm) || (iter <= 1) )   
        
        sty = 0.0;

        for i = 1:n

             t1 = xtemp(i);
             t2 = gtemp(i);
             pg(i) = t1 - t2;
             sty = sty + (t1-x(i))*(t2-g(i));
             x(i) = t1;
             g(i) = t2;
        end

        index=1;    
        pg = xtemp - gtemp;
        [pg, inform] = proj(n, pg, r, b, BL, BU);

        if ( inform > 0 )
             disp('Error in the projection!');
             flag = 6;   
             break;
        end
        
        pg = pg - xtemp;
        pgnorm = max(abs(pg));            
    else
        
        pg = xtemp - gtemp;
        [pg, inform] = proj(n, pg, r, b, BL, BU);  

        if ( inform > 0 )            
             disp('Error in the projection!');
             flag = 6;   
             break;
        end

        pg = pg - xtemp;
        pgnorm = max(abs(pg));   
 
        if ( sqrt(sts) > max (0.1*abs(ftemp)/pgnorm,1.0) )

              sty = 0.0;

              for i=1:n
                t1 = xtemp(i);
                t2 = gtemp(i);
                sty = sty + (t1-x(i))*(t2-g(i));  
                x(i)=t1;
                g(i)=t2;
              end
              
              index=1;    
             
        elseif ( sqrt(sts) <= min(0.1*abs(ftemp)/pgnorm,1.0) )

              sty = 0.0;
              yty = 0.0;

              for i=1:n
                t1 = xtemp(i);
                t2 = gtemp(i);
                y  = t2-g(i);
                sty = sty + (t1-x(i))*y;
                yty = yty + y*y;
                x(i)=t1;
                g(i)=t2;          
              end           

              cosine = abs(sty)/sqrt(sts*yty);

              if (cosine >= gamma)
                 index=1;
              end
        else
            x = xtemp;
            g = gtemp;   
        end
    end  
       
    if ( index == 1)

           ll = 0;

           if ( sty <= 0.0)
               
             if ( mm >= 6 )
                 xnorm = max(abs(x));
                 t1 = min(1.0/pgnorm, xnorm/pgnorm);
                 lambda = max(t1, lambda);
                 mm = 0;            
             end  
           else
             lambda = min(LMAX,max(LMIN,sts/sty));
             mm = 0;
           end
    end
      
    % finish computations of the initial stepsize

    f = ftemp;     
    ii= mod(iter,m)+1;
    lastfvalues(ii)= f;  
    
    disp([' It.: ' sprintf('%4i',iter) ' Obj.: ' sprintf('%20.15f',f) ...
          ' pgnorm: ' sprintf('%20.15f', pgnorm)]);
      
    if ( f <= fmin )
           xbest = x;
           pgnorm_best = pgnorm;
    end
end

x = xbest;
f = fmin; 
pgnorm = pgnorm_best;

%-----------------------------------------------








